@@ -698,8 +698,8 @@ class FITS_rec(np.recarray):
 
         if format.dtype.itemsize == 0:
             warnings.warn(
-                "Field {!r} has a repeat count of 0 in its format code, "
-                "indicating an empty field.".format(key)
+                f"Field {key!r} has a repeat count of 0 in its format code, "
+                "indicating an empty field."
             )
             return np.array([], dtype=format.dtype)
 
@@ -803,15 +803,15 @@ class FITS_rec(np.recarray):
 
         if raw_data is None:
             raise OSError(
-                "Could not find heap data for the {!r} variable-length "
-                "array column.".format(column.name)
+                f"Could not find heap data for the {column.name!r} variable-length "
+                "array column."
             )
 
         for idx in range(len(self)):
             offset = field[idx, 1] + self._heapoffset
             count = field[idx, 0]
 
-            if recformat.dtype == "a":
+            if recformat.dtype == "S":
                 dt = np.dtype(recformat.dtype + str(1))
                 arr_len = count * dt.itemsize
                 da = raw_data[offset : offset + arr_len].view(dt)
@@ -878,8 +878,8 @@ class FITS_rec(np.recarray):
         except ValueError as exc:
             indx = self.names.index(column.name)
             raise ValueError(
-                "{}; the header may be missing the necessary TNULL{} "
-                "keyword or the table contains invalid data".format(exc, indx + 1)
+                f"{exc}; the header may be missing the necessary TNULL{indx + 1} "
+                "keyword or the table contains invalid data"
             )
 
         return dummy
@@ -985,8 +985,8 @@ class FITS_rec(np.recarray):
                         test_overflow += bzero64
                     except OverflowError:
                         warnings.warn(
-                            "Overflow detected while applying TZERO{:d}. "
-                            "Returning unscaled data.".format(indx + 1)
+                            f"Overflow detected while applying TZERO{indx + 1:d}. "
+                            "Returning unscaled data."
                         )
                     else:
                         field = test_overflow
@@ -1110,7 +1110,7 @@ class FITS_rec(np.recarray):
             if isinstance(recformat, _FormatP):
                 # Irritatingly, this can return a different dtype than just
                 # doing np.dtype(recformat.dtype); but this returns the results
-                # that we want.  For example if recformat.dtype is 'a' we want
+                # that we want.  For example if recformat.dtype is 'S' we want
                 # an array of characters.
                 dtype = np.array([], dtype=recformat.dtype).dtype
 
@@ -1149,7 +1149,8 @@ class FITS_rec(np.recarray):
                 if _number and (_scale or _zero) and column._physical_values:
                     dummy = field.copy()
                     if _zero:
-                        dummy -= bzero
+                        # Cast before subtracting to avoid overflow problems.
+                        dummy -= np.array(bzero).astype(dummy.dtype, casting="unsafe")
                     if _scale:
                         dummy /= bscale
                     # This will set the raw values in the recarray back to
@@ -1249,7 +1250,7 @@ class FITS_rec(np.recarray):
         spans = self._coldefs.spans
         format = self._coldefs[col_idx].format
 
-        # The the index of the "end" column of the record, beyond
+        # The index of the "end" column of the record, beyond
         # which we can't write
         end = super().field(-1).itemsize
         starts.append(end + starts[-1])
