@@ -1322,10 +1322,10 @@ class UnitBase:
             for len_bases, composed, tunit in results:
                 if len_bases > min_length:
                     break
-                else:
-                    factored = composed * tunit
-                    if is_final_result(factored):
-                        subresults.add(factored)
+
+                factored = composed * tunit
+                if is_final_result(factored):
+                    subresults.add(factored)
 
             if len(subresults):
                 cached_results[key] = subresults
@@ -1975,7 +1975,7 @@ class UnrecognizedUnit(IrreducibleUnit):
     __reduce__ = object.__reduce__
 
     def __repr__(self):
-        return f"UnrecognizedUnit({str(self)})"
+        return f"UnrecognizedUnit({self})"
 
     def __bytes__(self):
         return self.name.encode("ascii", "replace")
