@@ -17,7 +17,12 @@ from astropy.units.quantity_helper.function_helpers import (
     TBD_FUNCTIONS,
     UNSUPPORTED_FUNCTIONS,
 )
-from astropy.utils.compat import NUMPY_LT_1_23, NUMPY_LT_1_24
+from astropy.utils.compat import (
+    NUMPY_LT_1_23,
+    NUMPY_LT_1_24,
+    NUMPY_LT_1_25,
+    NUMPY_LT_2_0,
+)
 
 needs_array_function = pytest.mark.xfail(
     not ARRAY_FUNCTION_ENABLED, reason="Needs __array_function__ support"
@@ -25,15 +30,32 @@ needs_array_function = pytest.mark.xfail(
 
 
 # To get the functions that could be covered, we look for those that
-# are wrapped.  Of course, this does not give a full list pre-1.17.
+# are in modules we care about and have been overridden.
 def get_wrapped_functions(*modules):
+    if NUMPY_LT_1_25:
+
+        def allows_array_function_override(f):
+            return (
+                hasattr(f, "__wrapped__")
+                and f is not np.printoptions
+                and not f.__name__.startswith("_")
+            )
+
+    else:
+        from numpy.testing.overrides import allows_array_function_override
+
     wrapped_functions = {}
     for mod in modules:
         for name, f in mod.__dict__.items():
-            if f is np.printoptions or name.startswith("_"):
-                continue
-            if callable(f) and hasattr(f, "__wrapped__"):
+            if callable(f) and allows_array_function_override(f):
+                # Indexing by just the name is easiest for test writing,
+                # but in numpy 2.0, there are 2 versions of diagonal and
+                # trace, one in the main namespace, and one in np.linalg.
+                # So, we distinguish those by adding the module name.
+                if name in wrapped_functions:
+                    name = mod.__name__.replace("numpy.", "") + "_" + name
                 wrapped_functions[name] = f
+
     return wrapped_functions
 
 
@@ -286,6 +308,7 @@ class TestCopyAndCreation(InvariantUnitTestSetup):
         copy = np.copy(a=self.q)
         assert_array_equal(copy, self.q)
 
+    @pytest.mark.skipif(not NUMPY_LT_2_0, reason="np.asfarray is removed in NumPy 2.0")
     @needs_array_function
     def test_asfarray(self):
         self.check(np.asfarray)
@@ -638,13 +661,13 @@ class TestUfuncReductions(InvariantUnitTestSetup):
     @pytest.mark.filterwarnings("ignore:`sometrue` is deprecated as of NumPy 1.25.0")
     def test_sometrue(self):
         with pytest.raises(TypeError):
-            np.sometrue(self.q)
+            np.sometrue(self.q)  # noqa: NPY003
 
     # NUMPY_LT_1_25
     @pytest.mark.filterwarnings("ignore:`alltrue` is deprecated as of NumPy 1.25.0")
     def test_alltrue(self):
         with pytest.raises(TypeError):
-            np.alltrue(self.q)
+            np.alltrue(self.q)  # noqa: NPY003
 
     def test_prod(self):
         with pytest.raises(u.UnitsError):
@@ -654,7 +677,7 @@ class TestUfuncReductions(InvariantUnitTestSetup):
     @pytest.mark.filterwarnings("ignore:`product` is deprecated as of NumPy 1.25.0")
     def test_product(self):
         with pytest.raises(u.UnitsError):
-            np.product(self.q)
+            np.product(self.q)  # noqa: NPY003
 
     def test_cumprod(self):
         with pytest.raises(u.UnitsError):
@@ -664,7 +687,7 @@ class TestUfuncReductions(InvariantUnitTestSetup):
     @pytest.mark.filterwarnings("ignore:`cumproduct` is deprecated as of NumPy 1.25.0")
     def test_cumproduct(self):
         with pytest.raises(u.UnitsError):
-            np.cumproduct(self.q)
+            np.cumproduct(self.q)  # noqa: NPY003
 
 
 class TestUfuncLike(InvariantUnitTestSetup):
@@ -676,9 +699,10 @@ class TestUfuncLike(InvariantUnitTestSetup):
         self.check(np.round)
 
     # NUMPY_LT_1_25
+    @pytest.mark.skipif(not NUMPY_LT_2_0, reason="np.round_ is removed in NumPy 2.0")
     @pytest.mark.filterwarnings("ignore:`round_` is deprecated as of NumPy 1.25.0")
     def test_round_(self):
-        self.check(np.round_)
+        self.check(np.round_)  # noqa: NPY003
 
     def test_around(self):
         self.check(np.around)
@@ -872,6 +896,12 @@ class TestReductionLikeFunctions(InvariantUnitTestSetup):
     def test_median(self):
         self.check(np.median)
 
+    def test_median_nan_scalar(self):
+        # See gh-12165; this dropped the unit in numpy < 1.22
+        data = [1.0, 2, np.nan, 3, 4] << u.km
+        result = np.median(data)
+        assert_array_equal(result, np.nan * u.km)
+
     @needs_array_function
     def test_quantile(self):
         self.check(np.quantile, 0.5)
@@ -943,6 +973,17 @@ class TestReductionLikeFunctions(InvariantUnitTestSetup):
         q3 = q1.value * u.cm
         assert not np.array_equal(q1, q3)
 
+    @pytest.mark.parametrize("equal_nan", [False, True])
+    def test_array_equal_nan(self, equal_nan):
+        q1 = np.linspace(0, 1, num=11) * u.m
+        q1[0] = np.nan
+        q2 = q1.to(u.cm)
+        result = np.array_equal(q1, q2, equal_nan=equal_nan)
+        assert result == equal_nan
+
+    def test_array_equal_incompatible_units(self):
+        assert not np.array_equal([1, 2] * u.m, [1, 2] * u.s)
+
     @needs_array_function
     def test_array_equiv(self):
         q1 = np.array([[0.0, 1.0, 2.0]] * 3) * u.m
@@ -951,6 +992,9 @@ class TestReductionLikeFunctions(InvariantUnitTestSetup):
         q3 = q1[0].value * u.cm
         assert not np.array_equiv(q1, q3)
 
+    def test_array_equiv_incompatible_units(self):
+        assert not np.array_equiv([1, 1] * u.m, [1] * u.s)
+
 
 class TestNanFunctions(InvariantUnitTestSetup):
     def setup_method(self):
@@ -976,8 +1020,15 @@ class TestNanFunctions(InvariantUnitTestSetup):
     def test_nanmean(self):
         self.check(np.nanmean)
 
-    def test_nanmedian(self):
-        self.check(np.nanmedian)
+    @pytest.mark.parametrize("axis", [None, 0, 1, -1])
+    def test_nanmedian(self, axis):
+        self.check(np.nanmedian, axis=axis)
+
+    def test_nanmedian_out(self):
+        out = np.empty_like(self.q)
+        o = np.nanmedian(self.q, out=out)
+        assert o is out
+        assert np.all(o == np.nanmedian(self.q))
 
     def test_nansum(self):
         self.check(np.nansum)
@@ -1110,6 +1161,7 @@ class TestVariousProductFunctions(metaclass=CoverageMeta):
 
 
 class TestIntDiffFunctions(metaclass=CoverageMeta):
+    @pytest.mark.filterwarnings("ignore:`trapz` is deprecated. Use `scipy.*")
     def test_trapz(self):
         y = np.arange(9.0) * u.m / u.s
         out = np.trapz(y)
@@ -1298,7 +1350,7 @@ class TestHistogramFunctions(metaclass=CoverageMeta):
         value_args=None,
         value_kwargs=None,
         expected_units=None,
-        **kwargs
+        **kwargs,
     ):
         """Check quanties are treated correctly in the histogram function.
         Test is done by applying ``function(*args, **kwargs)``, where
@@ -1903,6 +1955,7 @@ class TestSetOpsFcuntions(metaclass=CoverageMeta):
         self.check2(np.setdiff1d)
 
     @needs_array_function
+    @pytest.mark.filterwarnings("ignore:`in1d` is deprecated. Use `np.isin` instead.")
     def test_in1d(self):
         self.check2(np.in1d, unit=None)
         # Check zero is treated as having any unit.
@@ -2006,15 +2059,11 @@ class TestFFT(InvariantUnitTestSetup):
         self.check(np.fft.ifftshift)
 
 
-class TestLinAlg(metaclass=CoverageMeta):
+class TestLinAlg(InvariantUnitTestSetup, metaclass=CoverageMeta):
     def setup_method(self):
         self.q = (
-            np.array(
-                [[ 1.0, -1.0,  2.0],  # noqa: E201
-                 [ 0.0,  3.0, -1.0],  # noqa: E201
-                 [-1.0, -1.0,  1.0]]
-            ) << u.m
-        )  # fmt: skip
+            np.array([[1.0, -1.0, 2.0], [0.0, 3.0, -1.0], [-1.0, -1.0, 1.0]]) << u.m
+        )
 
     def test_cond(self):
         c = np.linalg.cond(self.q)
@@ -2233,6 +2282,17 @@ class TestLinAlg(metaclass=CoverageMeta):
         wx = np.linalg.eigvalsh(self.q.value) << self.q.unit
         assert_array_equal(w, wx)
 
+    if not NUMPY_LT_2_0:
+        # Numpy 2.0 added array-api compatible definitions of
+        # diagonal and trace to np.linalg. Since these have
+        # name conflicts with the main numpy namespace, they
+        # are tracked as linalg_diagonal and linalg_trace.
+        def test_linalg_diagonal(self):
+            self.check(np.linalg.diagonal)
+
+        def test_linalg_trace(self):
+            self.check(np.trace)
+
 
 class TestRecFunctions(metaclass=CoverageMeta):
     @classmethod
@@ -2323,18 +2383,17 @@ class TestRecFunctions(metaclass=CoverageMeta):
         assert np.array_equal(arr, self.q_pv)
         assert arr.unit == (u.km, u.km / u.s)
 
-    @pytest.mark.xfail
-    @pytest.mark.parametrize("flatten", [True, False])
-    def test_merge_arrays_nonquantities(self, flatten):
-        # Fails because cannot create quantity from structured array.
-        arr = rfn.merge_arrays((q_pv["p"], q_pv.value), flatten=flatten)
-
     def test_merge_array_nested_structure(self):
         # Merge 2-element tuples without flattening.
         arr = rfn.merge_arrays((self.q_pv, self.q_pv_t))
         assert_array_equal(arr["f0"], self.q_pv)
         assert_array_equal(arr["f1"], self.q_pv_t)
         assert arr.unit == ((u.km, u.km / u.s), ((u.km, u.km / u.s), u.s))
+        # For a structured array, all elements should be treated as dimensionless.
+        arr = rfn.merge_arrays((self.q_pv["p"], self.q_pv.value))
+        expected_value = rfn.merge_arrays((self.q_pv["p"].value, self.q_pv.value))
+        assert_array_equal(arr.value, expected_value)
+        assert arr.unit == u.Unit((self.q_pv["p"].unit, (u.one, u.one)))
 
     def test_merge_arrays_flatten_nested_structure(self):
         # Merge 2-element tuple, flattening it.
@@ -2345,6 +2404,13 @@ class TestRecFunctions(metaclass=CoverageMeta):
         assert_array_equal(arr["vv"], self.q_pv_t["pv"]["vv"])
         assert_array_equal(arr["t"], self.q_pv_t["t"])
         assert arr.unit == (u.km, u.km / u.s, u.km, u.km / u.s, u.s)
+        # For a structured array, all elements should be treated as dimensionless.
+        arr = rfn.merge_arrays((self.q_pv["p"], self.q_pv.value), flatten=True)
+        expected_value = rfn.merge_arrays(
+            (self.q_pv["p"].value, self.q_pv.value), flatten=True
+        )
+        assert_array_equal(arr.value, expected_value)
+        assert arr.unit == u.Unit((self.q_pv["p"].unit, u.one, u.one))
 
     def test_merge_arrays_asrecarray(self):
         with pytest.raises(ValueError, match="asrecarray=True is not supported."):
